#ifndef _TARTPPAC_H_
#define _TARTPPAC_H_

#include <TObjArray.h>
#include "TArtCalibPID.h"

class TArtBigRIPSParameters;
class TArtPPACPara;
class TArtMapper;

class TArtPPACArray : public TObjArray {

 public:
  TArtPPACArray() {}
  TArtPPACArray(TArtBigRIPSParameters *para) {LoadSetup(para);}
  virtual ~TArtPPACArray(){}

  virtual void LoadSetup(TArtBigRIPSParameters *para);
  virtual void LoadRawDataMap(TArtMapper *mapper);
  virtual void ClearData();
  virtual void ReconstructData(); // raw value -> physics value

  ClassDef(TArtPPACArray,1);

};

class TArtPPAC : public TNamed {
 
 public:
  TArtPPAC(TArtPPACPara *p);
  virtual ~TArtPPAC();

  void LoadRawDataMap(TArtMapper *mapper);
  void ClearData(); 
  void ReconstructData(); // raw value -> physics value
  bool IsReconstructed(){ return fReconstructed; }

  // function to set raw data
  void  SetRawData(int x1, int x2, int y1, int y2, int a){fTX1Raw = x1; fTX2Raw = x2; fTY1Raw = y1; fTY2Raw = y2; fTARaw = a;}
  void  SetTX1Raw(int val){fTX1Raw = val;}
  void  SetTX2Raw(int val){fTX2Raw = val;}
  void  SetTY1Raw(int val){fTY1Raw = val;}
  void  SetTY2Raw(int val){fTY2Raw = val;}
  void  SetTARaw(int val){fTARaw = val;}

  unsigned int GetTX1Raw(){return fTX1Raw;}
  unsigned int GetTX2Raw(){return fTX2Raw;}
  unsigned int GetTY1Raw(){return fTY1Raw;}
  unsigned int GetTY2Raw(){return fTY2Raw;}
  unsigned int GetTARaw(){return fTARaw;}

  bool  IsFiredX(){return fFiredX;}  // Detector fired in x
  bool  IsFiredY(){return fFiredY;}  // Detector fired in y
  bool  IsFired(){return fFiredX && fFiredY;}  // Detector fired in x AND y

  Int_t GetFpl();
  Double_t GetXZPos();
  Double_t GetYZPos();
  double GetX(){return fX;}
  double GetY(){return fY;}

  // calibration parameter
  Int_t GetID();
  TString * GetDetectorName();
  TArtPPACPara * GetParameter(){return para;}

 private:
  unsigned int fTX1Raw; // raw data 
  unsigned int fTX2Raw;
  unsigned int fTY1Raw;	
  unsigned int fTY2Raw;
  unsigned int fTARaw;

  bool    fFiredX; 
  bool    fFiredY; 
  double  fX; 
  double  fY; 

  bool    fReconstructed;

  TArtPPACPara *para;

  ClassDef(TArtPPAC,1);

};
#endif
