#ifndef _TARTPLASTIC_H_
#define _TARTPLASTIC_H_

#include <TObjArray.h>
#include "TArtCalibPID.h"

class TArtBigRIPSParameters;
class TArtPlasticPara;
class TArtMapper;

class TArtPlasticArray : public TObjArray {

 public:
  TArtPlasticArray() {}
  TArtPlasticArray(TArtBigRIPSParameters *para) {LoadSetup(para);}
  virtual ~TArtPlasticArray(){}

  virtual void LoadSetup(TArtBigRIPSParameters *para);
  virtual void LoadRawDataMap(TArtMapper *mapper);
  virtual void ClearData();
  virtual void ReconstructData(); // raw value -> physics value

  ClassDef(TArtPlasticArray,1);

};

class TArtPlastic : public TNamed {

 public:
  TArtPlastic(TArtPlasticPara *p);
  virtual ~TArtPlastic();

  void   LoadRawDataMap(TArtMapper *mapper);
  void   ClearData();
  void   ReconstructData(); // raw value -> physics value
  Bool_t IsReconstructed(){ return fReconstructed; }

  // function to set raw data
  void  SetTLRaw(int val){fTLRaw = val;}
  void  SetTRRaw(int val){fTRRaw = val;}

  // function to get raw data
  unsigned int GetTLRaw(){fTLRaw;}
  unsigned int GetTRRaw(){fTRRaw;}
 
  // Reconstructed values
  Double_t GetTime(){return fTime;}
  Bool_t  IsFired(){return fFired;}

  // calibration parameter
  Int_t     GetID();
  Int_t     GetFpl();
  void      SetZpos(Double_t z){zposition = z;}
  Double_t  GetZpos(){return zposition;}
  Double_t  GetZoffset();
  TString * GetDetectorName();
  TArtPlasticPara * GetParameter(){return para;}

 private:
  unsigned int fTLRaw;  
  unsigned int fTRRaw;
  unsigned int fQLRaw;  
  unsigned int fQRRaw;

  Bool_t     fReconstructed;
  Double_t   zposition;
  Double_t   fTime;
  Bool_t     fFired;          // Check if the plastic is fired.

  TArtPlasticPara *para;

  ClassDef(TArtPlastic,1);

};
#endif

