#include "TArtPPAC.h"
#include "TArtPlastic.h"
#include "TArtBigRIPSParameters.h"
#include "TArtFocalPlane.h"
#include "TArtFocalPlanePara.h"
#include "TArtCore.h"

#include  <TMath.h>
#include  <TMatrixD.h>
#include  <TList.h>

//__________________________________________________________
void TArtFocalPlaneArray::LoadSetup(TArtBigRIPSParameters *para) 
{

  Clear();
  TList *listOfFocalPlanePara = (TList *)para->GetListOfFocalPlanePara();
  TIter next(listOfFocalPlanePara);
  TArtFocalPlanePara *p;
  while(p = (TArtFocalPlanePara*)next()) {
    Add(new TArtFocalPlane(p));
  }

}

//__________________________________________________________
void TArtFocalPlaneArray::LoadPPACMap(TArtPPACArray *ppacs) 
{
  for(int i=0;i<GetEntries();i++)
    ((TArtFocalPlane*)At(i))->LoadPPACMap(ppacs);
  loadppacmap = true;
}

//__________________________________________________________
void TArtFocalPlaneArray::LoadPlasticMap(TArtPlasticArray *plas) 
{
  for(int i=0;i<GetEntries();i++)
    ((TArtFocalPlane*)At(i))->LoadPlasticMap(plas);
  loadplmap = true;
}

//__________________________________________________________
void TArtFocalPlaneArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtFocalPlane*)At(i))->ClearData();
}

//__________________________________________________________
void TArtFocalPlaneArray::RecoBeamPos()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtFocalPlane*)At(i))->RecoBeamPos();
}

//__________________________________________________________
TArtFocalPlane::TArtFocalPlane(TArtFocalPlanePara *p) : para(p)
{
  opt_vector.ResizeTo(5);
  ppac_array = new TArtPPACArray();
  pla_array = new TArtPlasticArray();
}

//__________________________________________________________
TArtFocalPlane::~TArtFocalPlane() 
{
  delete ppac_array;
  delete pla_array;
}

//__________________________________________________________
void TArtFocalPlane::LoadPPACMap(TArtPPACArray *ppacs)
{
  int fpl = para->GetFpl();
  for(int i=0;i<ppacs->GetEntries();i++){
    TArtPPAC * ppac = (TArtPPAC *)ppacs->At(i);
    if( ppac->GetFpl() == fpl )
      ppac_array->Add(ppac);
  }
  for(int i=0;i<ppac_array->GetEntries();i++)
    TArtCore::Info(__FILE__,"PPAC %s used for fpl%d",((TArtPPAC*)ppac_array->At(i))->GetDetectorName()->Data(),fpl);

  return;
} 

//__________________________________________________________
void TArtFocalPlane::LoadPlasticMap(TArtPlasticArray *plas)
{
  int fpl = GetFpl();
  for(int i=0;i<plas->GetEntries();i++){
    TArtPlastic * pl = (TArtPlastic *)plas->At(i);
    if( pl->GetFpl() == fpl ){
      pla_array->Add(pl);
      pl->SetZpos( GetZpos() + pl->GetZoffset() );
      TArtCore::Info(__FILE__,"zpos of Plastic %s is %f + %f",pl->GetDetectorName()->Data(), (float)GetZpos(), (float)pl->GetZoffset() );
    }
  }

  return;
} 

//__________________________________________________________
void TArtFocalPlane::ClearData(){

  for(int i=0;i<5;i++)
    opt_vector(i) = -9999;
  nfired_ppacx = 0;
  nfired_ppacy = 0;
  return;

}

//__________________________________________________________
void TArtFocalPlane::RecoBeamPos()  {
 
  // on the assumption of linear track, the track is reconstructed analytically.
  // X = a + b * Z;

  TMatrixD xvec(2,1); xvec.Zero();
  TMatrixD yvec(2,1); yvec.Zero();
  TMatrixD xmat(2,2); xmat.Zero();
  TMatrixD ymat(2,2); ymat.Zero();

  double sum_zx[2] = {0,0};
  double sum_x[2] = {0,0};
  double sum_z[2] = {0,0};
  double sum_zz[2] = {0,0};
  double x, y, z_x, z_y;

  int nppac = ppac_array->GetEntries();
  double pZ_f = para->GetFocusPos();
  TArtPPAC *ppac = NULL;

  if(nppac<=0){
  }
  else if(nppac==1) {
    ppac = (TArtPPAC *)ppac_array->At(0);
    opt_vector(0) = ppac->GetX();
    opt_vector(2) = ppac->GetY();
    if(ppac->IsFiredX()) nfired_ppacx = 1;
    if(ppac->IsFiredY()) nfired_ppacy = 1;
  }
  else { 
    for(int i=0;i<nppac;i++){
      ppac = (TArtPPAC *)ppac_array->At(i);
      x = ppac->GetX();
      y = ppac->GetY();
      z_x = ppac->GetXZPos();
      z_y = ppac->GetYZPos();
      if(ppac->IsFiredX()){
	xvec(0,0) += z_x * x; // b(1) in rayfit
	xvec(1,0) += x; // b(2) in rayfit
	xmat(0,1) += z_x; // a(1,2) in rayfit
	xmat(1,0) += z_x; // a(1,2) in rayfit
	xmat(0,0) += z_x * z_x;   // a(1,1) in rayfit
	xmat(1,1) ++;  // a(2,2) in rayfit
	nfired_ppacx ++;  // a(2,2) in rayfit
      }
      if(ppac->IsFiredY()){
	yvec(0,0) += z_y * y; // b(1) in rayfit
	yvec(1,0) += y; // b(2) in rayfit
	ymat(0,1) += z_y; // a(1,2) in rayfit
	ymat(1,0) += z_y; // a(1,2) in rayfit
	ymat(0,0) += z_y * z_y;   // a(1,1) in rayfit
	ymat(1,1) ++;  // a(2,2) in rayfit
	nfired_ppacy ++;  // a(2,2) in rayfit
      }
    }

    // determine a and b in x = a + b * z (i.e.); 
    if(nfired_ppacx>1){
      TMatrixD rxvec = xmat.Invert() * xvec;
      opt_vector(0) = rxvec(1,0);
      opt_vector(1) = TMath::ATan(rxvec(0,0)) * 1000;
    }
    if(nfired_ppacy>1){
      TMatrixD ryvec = ymat.Invert() * yvec;
      opt_vector(2) = ryvec(1,0);
      opt_vector(3) = TMath::ATan(ryvec(0,0)) * 1000; // the value is given in mrad.;
    }

  } 

  //if only one fired the angle is set to zero
 
}

//__________________________________________________________
Int_t TArtFocalPlane::GetID() {
  return para->GetID();
}
//__________________________________________________________
Double_t TArtFocalPlane::GetZpos(){
  return para->GetZpos();
}
//__________________________________________________________
Int_t TArtFocalPlane::GetFpl(){
  return para->GetFpl();
}
